package com.example.ektappv1;

import androidx.activity.OnBackPressedCallback;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.webkit.CookieManager;
import android.webkit.CookieSyncManager;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.CollationKey;
import java.util.Set;

public class MainActivity extends AppCompatActivity {

    private WebView wbEKT;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        wbEKT = findViewById(R.id.wbEKT);
        wbEKT = configureWebView(wbEKT);


        /*
        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            String postData = null;
            try {
                postData = "username=" + URLEncoder.encode(extras.getString("user"), "UTF-8") +
                "&password=" + URLEncoder.encode(extras.getString("password"), "UTF-8") ;
                Log.v("JUAN", "post: "+postData);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            wbEKT.postUrl(Globals.urlLoginEKT,postData.getBytes());
        }*/


        wbEKT.loadUrl(Globals.urlEKT);

    }

    protected WebView configureWebView(WebView webView){

        webView.setWebViewClient(generateWebClient());
        setWebSettings(webView.getSettings());
        Historic.initHistoric();

        return webView;
    }

    protected void setWebSettings(WebSettings webSettings){
        webSettings.setJavaScriptEnabled(true);

        webSettings.setRenderPriority(WebSettings.RenderPriority.HIGH);
        webSettings.setCacheMode(WebSettings.LOAD_CACHE_ELSE_NETWORK);
        webSettings.setAppCacheEnabled(true);
        webSettings.setDomStorageEnabled(true);
        webSettings.setLayoutAlgorithm(WebSettings.LayoutAlgorithm.NARROW_COLUMNS);
        webSettings.setUseWideViewPort(true);
        webSettings.setSaveFormData(true);
        webSettings.setAppCachePath(getCacheDir().getAbsolutePath());
    }

    private WebViewClient generateWebClient(){
        return new WebViewClient() {
            @Override
            public void onPageStarted(WebView view, String url, Bitmap favicon) {
                super.onPageStarted(view, url, favicon);

                String urlBase = Historic.getBaseURL(url);
                checkUrl(urlBase);

                Log.v("JUAN", "count: "+Globals.countLoopingUrls+"URL LOADED: "+url);
                Log.v("JUAN","COOKIE: "+CookieManager.getInstance().getCookie("https://app.ektproject.eu"));
            }

            @Override
            public void onPageFinished(WebView view, String url) {
                super.onPageFinished(view, url);

                Log.v("JUAN","COOKIE: "+CookieManager.getInstance().getCookie("https://app.ektproject.eu"));

                if(Historic.getBaseURL(url).equals(Globals.urlLoginEKT)){
                    Uri uri = Uri.parse(url);
                    String server = uri.getAuthority();
                    String path = uri.getPath();
                    String protocol = uri.getScheme();
                    Set<String> args = uri.getQueryParameterNames();
                    String username = uri.getQueryParameter("username");
                    String password = uri.getQueryParameter("password");

                    Log.v("JUAN","LOGIN "+username+" "+password);
                    CookieManager.getInstance().getCookie(Globals.urlLoginEKT);
                }

            }
        };
    }

    private void checkUrl(String urlBase) {
        //to avoid repeated urls in the historic
        if(!Globals.lastPageHistoric.equals(urlBase) && !urlBase.equals("")){
            Historic.setURLinHistoric(urlBase);
            Globals.countLoopingUrls = 0;
        }
        else{
            Globals.countLoopingUrls++;
            if(Globals.countLoopingUrls > 2){
                wbEKT.stopLoading();
                wbEKT.clearCache(true);
                wbEKT.loadUrl(Globals.urlEKT);
                Globals.countLoopingUrls = 0;
            }
        }

        switch (urlBase){
            case Globals.urlTalkNC:
                //We stop the request to NC to open the app, delete request to NC in historic and reload the last page to prevent stopping webview functionality
                wbEKT.stopLoading();
                Historic.unsetLastPage();
                wbEKT.loadUrl(Globals.lastPageHistoric);

                if(!Functions.isPackageInstalled(Globals.packageNameNC, getApplicationContext().getPackageManager())){
                    dialogNCTalkNotInstalled();
                }
                else{
                    Context ctx = MainActivity.this;
                    Intent i = ctx.getPackageManager().getLaunchIntentForPackage(Globals.packageNameNC);
                    ctx.startActivity(i);
                }
                break;

            case Globals.urlLogoutEKT:
                logout();
                break;
        }
    }

    private void logout() {
        wbEKT.clearCache(true);
        Historic.clearCookies(MainActivity.this);
    }


    private void dialogNCTalkNotInstalled() {
        AlertDialog alertDialog = new AlertDialog.Builder(MainActivity.this).create();
        alertDialog.setTitle(getString(R.string.nc_talk_not_installed));
        alertDialog.setMessage(getString(R.string.check_nc_talk_installation));
        alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, getString(R.string.accept), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
                try{
                    startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=" + Globals.packageNameNC)));
                }catch (ActivityNotFoundException exception)
                {
                    // app store is not available
                    Log.v("JUAN", "PLAY STORE NO INSTALADO");
                    Toast.makeText(MainActivity.this, getString(R.string.google_play_not_installed), Toast.LENGTH_SHORT).show();
                }
            }
        });

        alertDialog.show();
    }

    @Override
    public void onBackPressed() {
        Log.v("JUAN","ON BACK");
        if(Globals.listHistoric.size() > 0){
            Historic.unsetLastPage();
            wbEKT.loadUrl(Globals.lastPageHistoric);
        }
        else
            super.onBackPressed();
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.v("JUAN","ON RESUME");
        if(Globals.listHistoric.size() > 0)
            wbEKT.loadUrl(Globals.lastPageHistoric);
    }
}